require 'soar/policy/access_manager/model_provider/service_registry'
require 'soar/policy/access_manager/model'
require 'jsender'
require 'uri'
require 'ostruct'

module Soar
  module Policy
    module AccessManager
      module Test
        module OrchestrationProvider
          class ServiceRegistry 

            class Services
              include Jsender

              def initialize()
                @policy_host = ENV['POLICY_HOST'] || 'localhost:8080'
                @meta = {
                  'service_identifier1' => {
                    'policy' => 'allow'
                  },
                  'service_identifier2' => {
                    'policy' => 'deny'
                  }
                }
              end

              def service_by_name(policy)
                case policy
                  when 'allow' # allow policy
                    success_data({
                      "services" => [[{}, { "uris" => [[{}, { "access_point" => "http://#{@policy_host}/allow" }]]}]]
                    })
                  when 'deny' # deny policy
                    success_data({
                      "services" => [[{}, { "uris" => [[{}, { "access_point" => "http://#{@policy_host}/deny" }]]}]]
                    })
                  else # no policy
                    success_data({
                      "services" => [[{}, { "uris" => [[{}, { "access_point" => "http://#{@policy_host}/" }]]}]]
                    })
                end
              end

              def meta_for_service(service_identifier)
                @meta[service_identifier]
              end

            end

            class Stub 
              include Jsender

              attr_accessor :services

              def initialize(services)
                @services = services
              end

            end

            def initialize
              @resource_identifier = 'resource_identifier1'
              @request = {
                params: {},
                authentication_identifier: 'authentication_identifier1'
              }
            end

            def grant_access
              @service_identifier = 'service_identifier1'
            end

            def deny_access
              @service_identifier = 'service_identifier2'
            end

            def no_policy
              @service_identifier = 'service_identifier3'
            end

            def notification
              @response['data']['notifications']
            end

            def authorized?
              service_registry = Stub.new(Services.new)
              model_provider = Soar::Policy::AccessManager::ModelProvider::ServiceRegistry.new(service_registry)
              model = Soar::Policy::AccessManager::Model.new(model_provider)
              @response = model.authorized?(service_identifier: @service_identifier, resource_identifier: @resource_identifier, request: @request)
            end

            def authorized
              @response['data']['approved']
            end

          end
        end
      end
    end
  end
end
